# -*- coding: utf-8 -*-
from otterplot.configuration.iconfiguration import IConfiguration


class CConfigDefaultLatex(IConfiguration):
    def __init__(self,**kwargs) -> None:
        super().__init__()
