# -*- coding: utf-8 -*-
r"""
This module contains the constants for the fontsizes prescribed by the physical review b format.
"""
# ======= options for figure ========
FIGUREWIDTH_NARROW = 3.38583
FIGUREWIDTH_BROAD = 3.38583 * 2
# ======= options for ticks =========
FONTSIZEMAJORTICKS = 8
TICKWIDTH_MAJOR = 1.5
TICKWIDTH_MINOR = 1.0
TICKLENGTH_MAJOR = 5.0
TICKLENGTH_MINOR = 3.0
TICKDIRECTION = 'in'
# ======= options fontsizes =======
FONTSIZEXYLABEL = 8